"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ModeratorError_1 = __importDefault(require("../ModeratorError"));
const ModeratorErrors_json_1 = __importDefault(require("../ModeratorErrors.json"));
/**
 * Moderator Guilds Blacklist Manager
*/
class GuildBlacklistManager {
    client;
    moderator;
    errors;
    constructor(client, moderator) {
        /**
         * Moderator Errors JSON
         *
         * @type {Object}
         * @private
        */
        this.errors = ModeratorErrors_json_1.default;
        if (!client)
            throw new ModeratorError_1.default(this.errors.requireClient);
        if (!moderator)
            throw new ModeratorError_1.default(this.errors.requireModerator);
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
        */
        this.client = client;
        /**
         * Discord Moderator
         *
         * @type {Moderator}
         * @private
        */
        this.moderator = moderator;
        client.on('guildMemberAdd', async (member) => {
            if (moderator.options.blacklistManager) {
                const guild = member.guild;
                const guildBlocks = moderator.db.blacklist.get(guild.id);
                const reason = 'Discovered on the Blacklist by Moderator!';
                if (guildBlocks && guildBlocks.length > 0) {
                    const fetchUser = guildBlocks.find(guildBlock => guildBlock.userID === member.id);
                    if (fetchUser) {
                        switch (moderator.options.blacklistConfig.punishment) {
                            case 'ban': {
                                if (!moderator.utils.fetchClientPermissions(guild, ['BanMembers']))
                                    throw new ModeratorError_1.default(this.errors.MissingAccess);
                                if (moderator.utils.fetchMemberPermissions(member, ['Administrator', 'BanMembers']))
                                    throw new ModeratorError_1.default(this.errors.MissingAccess);
                                await moderator.punishments.ban(member, client.user.id, reason);
                                break;
                            }
                            case 'kick': {
                                if (!moderator.utils.fetchClientPermissions(guild, ['KickMembers']))
                                    throw new ModeratorError_1.default(this.errors.MissingAccess);
                                if (moderator.utils.fetchMemberPermissions(member, ['Administrator', 'KickMembers']))
                                    throw new ModeratorError_1.default(this.errors.MissingAccess);
                                await moderator.punishments.kick(member, client.user.id, reason);
                                break;
                            }
                        }
                    }
                }
            }
        });
    }
    /**
     * Method for issuing a lock to a user on the server
     * @param {String} targetID Discord User ID
     * @param {String} guildID Discord Guild ID
     * @param {String} author Block author ID
     * @param {String} reason Block reason
     *
     * @returns {Promise<{ status: Boolean, data: BlockData }>} Returns the lock issuance status and information about it
    */
    add(targetID, guildID, author, reason) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            if (!author)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'author')));
            const guild = this.client.guilds.cache.get(guildID);
            if (!guild)
                return reject(new ModeratorError_1.default(this.errors.guildNotFound.replace('{gID}', guildID)));
            const guildBlocks = this.moderator.db.blacklist.get(guild.id);
            const fetchUser = this.moderator.utils.fetchMember(guild, targetID);
            const blockData = { id: guildBlocks?.length ? guildBlocks?.length + 1 : 1, userID: fetchUser.status ? fetchUser.user.id : targetID, guildID: guild.id, currentTime: Date.now(), author: author, reason: reason || null };
            if (!guildBlocks) {
                this.moderator.db.blacklist.set(guild.id, [blockData]);
                return resolve({ status: true, data: blockData });
            }
            else {
                const searchBlock = guildBlocks.find(guildBlock => guildBlock.userID === targetID);
                if (searchBlock)
                    return reject(new ModeratorError_1.default(this.errors.blacklistManager.add.userBlocked.replace('{uID}', targetID).replace('{gID}', guild.id)));
                this.moderator.db.blacklist.push(guild.id, blockData);
                return resolve({ status: true, data: blockData });
            }
        });
    }
    /**
     * Method for checking if a user has a lock on the server
     * @param {String} targetID Discord User ID
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<{ status: Boolean, data: BlockData }>} Returns the validation status and lock information (if any)
    */
    fetch(targetID, guildID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guild = this.client.guilds.cache.get(guildID);
            if (!guild)
                return reject(new ModeratorError_1.default(this.errors.guildNotFound.replace('{gID}', guildID)));
            const guildBlocks = this.moderator.db.blacklist.get(guild.id);
            if (!guildBlocks || guildBlocks && !guildBlocks.length)
                return resolve({ status: false, data: null });
            const searchBlock = guildBlocks.find(block => block.userID === targetID);
            if (!searchBlock)
                return resolve({ status: true, data: null });
            else
                return resolve({ status: true, data: searchBlock });
        });
    }
    /**
     * Method to get user locks on all bot servers
     * @param {String} targetID Discord User ID
     *
     * @returns {Promise<{ status: Boolean, fetchUser: Boolean, data: Array<BlockData> }>} Returns the status of the search for locks and an array with them
    */
    fetchAll(targetID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            let targetBlocks = [];
            this.moderator.db.blacklist.forEach(guildBlocks => {
                const searchTargetData = guildBlocks.find(guildBlock => guildBlock.userID === targetID);
                if (searchTargetData)
                    targetBlocks.push(searchTargetData);
            });
            return resolve({ status: true, fetchUser: true, data: targetBlocks.length ? targetBlocks : null });
        });
    }
    /**
     * Method to remove a user lock on the server
     * @param {String} targetID Discord user ID
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<Boolean>} Returns the user lock removal status
    */
    remove(targetID, guildID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guild = this.client.guilds.cache.get(guildID);
            if (!guild)
                return reject(new ModeratorError_1.default(this.errors.guildNotFound.replace('{gID}', guildID)));
            const guildBlocks = this.moderator.db.blacklist.get(guild.id);
            if (!guildBlocks || guildBlocks && !guildBlocks.length)
                return resolve(false);
            const searchBlock = guildBlocks.find(guildBlock => guildBlock.userID === targetID);
            if (!searchBlock)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.remove.userNotBlocked.replace('{uID}', targetID).replace('{gID}', guildID)));
            const filteredBlocks = guildBlocks.filter(guildBlock => guildBlock.userID != targetID);
            this.moderator.db.blacklist.set(guildID, filteredBlocks);
            return resolve(true);
        });
    }
    /**
     * Method to clear all locks on the server
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<Boolean>} Returns the cleanup status
    */
    clearGuild(guildID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guildBlocks = this.moderator.db.blacklist.get(guildID);
            if (!guildBlocks || guildBlocks && !guildBlocks.length)
                return resolve(false);
            this.moderator.db.blacklist.delete(guildID);
            return resolve(true);
        });
    }
    /**
     * Method for completely clearing locks on all servers
     *
     * @returns {Promise<Boolean>} Returns the cleanup status
    */
    clearAll() {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.blacklistManager)
                return reject(new ModeratorError_1.default(this.errors.blacklistManager.disabled));
            const dataSize = this.moderator.db.blacklist.size;
            if (!dataSize)
                return resolve(false);
            this.moderator.db.blacklist.deleteAll();
            return resolve(true);
        });
    }
}
module.exports = GuildBlacklistManager;
