"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ms_1 = __importDefault(require("../modules/ms"));
const ModeratorError_1 = __importDefault(require("../ModeratorError"));
const ModeratorErrors_json_1 = __importDefault(require("../ModeratorErrors.json"));
/**
 * Moderator Mutes Manager
*/
class MutesManager {
    client;
    moderator;
    errors;
    constructor(client, moderator) {
        /**
         * Moderator Errors JSON
         *
         * @type {Object}
         * @private
        */
        this.errors = ModeratorErrors_json_1.default;
        if (!client)
            throw new ModeratorError_1.default(this.errors.requireClient);
        if (!moderator)
            throw new ModeratorError_1.default(this.errors.requireModerator);
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
        */
        this.client = client;
        /**
         * Discord Moderator
         *
         * @type {Moderator}
         * @private
        */
        this.moderator = moderator;
        client.on('guildMemberAdd', async (member) => {
            if (this.moderator.options.muteManager) {
                const guild = member.guild;
                if (!this.moderator.utils.fetchClientPermissions(guild, ['ManageRoles', 'KickMembers']))
                    throw new ModeratorError_1.default(this.errors.MissingPermissions);
                const guildMutes = this.moderator.db.mutes.get(guild.id);
                if (guildMutes && guildMutes.length) {
                    const searchTargetData = guildMutes.find(guildMute => guildMute.userID === member.id);
                    if (!searchTargetData)
                        return;
                    const muteRolePosition = member.guild.roles.cache.get(searchTargetData.roleID).position;
                    const clientRolePosition = member.guild.members.cache.get(client.user.id).roles.highest.position;
                    if (muteRolePosition >= clientRolePosition)
                        throw new ModeratorError_1.default(this.errors.MissingAccess);
                    await member.roles.add(searchTargetData.roleID).catch((error) => { return; });
                }
            }
        });
        moderator.on('ready', (client) => {
            if (moderator.options.muteManager) {
                setInterval(async () => {
                    moderator.db.mutes.forEach(async (guildMutes) => {
                        if (guildMutes.length <= 0)
                            return;
                        for (let i = 0; i < guildMutes.length; i++) {
                            const currentTime = Date.now();
                            const muteTime = guildMutes[i].time;
                            const muteTimeStart = guildMutes[i].currentTime;
                            if (muteTime != null) {
                                if (currentTime - muteTimeStart > muteTime) {
                                    const guild = this.client.guilds.cache.get(guildMutes[i].guildID);
                                    if (guild) {
                                        const target = guild.members.cache.get(guildMutes[i].userID);
                                        if (target) {
                                            if (!target.roles.cache.has(guildMutes[i].roleID)) {
                                                const filteredGuildMutes = guildMutes.filter(guildMute => guildMute.userID != target.id && guildMute.currentTime != muteTimeStart);
                                                moderator.db.mutes.set(guild.id, filteredGuildMutes);
                                                moderator.emit('muteEnded', guildMutes[i]);
                                            }
                                            else {
                                                const muteRolePosition = guild.roles.cache.get(guildMutes[i].roleID)?.position || null;
                                                const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
                                                if (muteRolePosition != null) {
                                                    if (muteRolePosition >= clientRolePosition)
                                                        return new ModeratorError_1.default(this.errors.MissingAccess);
                                                    const filteredGuildMutes = guildMutes.filter(guildMute => guildMute.userID != target.id && guildMute.currentTime != muteTimeStart);
                                                    moderator.db.mutes.set(guild.id, filteredGuildMutes);
                                                    await target.roles.remove(guildMutes[i].roleID).catch((error) => { return; });
                                                    moderator.emit('muteEnded', guildMutes[i]);
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    });
                }, ((0, ms_1.default)(moderator.options.muteConfig.interval)));
            }
        });
    }
    /**
     * Method for issuing a mute to a user
     * @param {GuildMember} target Discord GuildMember
     * @param {String} channelID Mute Channel ID
     * @param {String} roleID Mute Role ID
     * @param {String} reason Mute Reason
     *
     * @returns {Promise<{ status: Boolean, data: MuteData }>} Mute status and information about it
    */
    add(target, channelID, roleID, reason) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            if (!channelID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'channelID')));
            if (!roleID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'roleID')));
            const guild = target.guild;
            const guildMutes = this.moderator.db.mutes.get(guild.id);
            const muteRolePosition = guild.roles.cache.get(roleID).position;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            const muteData = { userID: target.id, guildID: guild.id, channelID: channelID, roleID: roleID, time: null, currentTime: Date.now(), reason: reason || null };
            if (!this.moderator.utils.fetchClientPermissions(guild, ['ManageRoles']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (this.moderator.utils.fetchMemberPermissions(target, ['Administrator', 'KickMembers', 'BanMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (muteRolePosition >= clientRolePosition)
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (!guildMutes || guildMutes === null) {
                this.moderator.db.mutes.set(guild.id, [muteData]);
                await target.roles.add(roleID).catch((error) => { return; });
                this.moderator.emit('addMute', muteData);
                return resolve({ status: true, data: muteData });
            }
            else {
                const searchTargetData = guildMutes.find(guildMute => guildMute.userID === target.id);
                if (searchTargetData)
                    return reject(new ModeratorError_1.default(this.errors.muteManager.add.userAlreadyMuted.replace('{ID}', target.id)));
                guildMutes.push(muteData);
                this.moderator.db.mutes.set(guild.id, guildMutes);
                await target.roles.add(roleID).catch((error) => { return; });
                this.moderator.emit('addMute', muteData);
                return resolve({ status: true, data: muteData });
            }
        });
    }
    /**
     * Method for issuing a temporary mute to a user
     * @param {GuildMember} target Discord GuildMember
     * @param {String} channelID Mute Channel ID
     * @param {String} roleID Mute Role ID
     * @param {String} time Mute time
     * @param {String} reason Mute Reason
     *
     * @returns {Promise<{ status: Boolean, data: MuteData }>} Mute status and information about it
    */
    temp(target, channelID, roleID, time, reason) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            if (!channelID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'channelID')));
            if (!roleID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'roleID')));
            if (!(0, ms_1.default)(time))
                return reject(new ModeratorError_1.default(this.errors.muteManager.temp.invalidValue));
            const guild = target.guild;
            const guildMutes = this.moderator.db.mutes.get(guild.id);
            const muteRolePosition = guild.roles.cache.get(roleID).position;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            const muteData = { userID: target.id, guildID: guild.id, channelID: channelID, roleID: roleID, time: (0, ms_1.default)(time), currentTime: Date.now(), reason: reason || null };
            if (!this.moderator.utils.fetchClientPermissions(guild, ['ManageRoles']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (this.moderator.utils.fetchMemberPermissions(target, ['Administrator', 'KickMembers', 'BanMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (muteRolePosition >= clientRolePosition)
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (!guildMutes || guildMutes === null) {
                this.moderator.db.mutes.set(guild.id, [muteData]);
                await target.roles.add(roleID).catch((error) => { return; });
                this.moderator.emit('addMute', muteData);
                return resolve({ status: true, data: muteData });
            }
            else {
                const searchTargetData = guildMutes.find(guildMute => guildMute.userID === target.id);
                if (searchTargetData)
                    return reject(new ModeratorError_1.default(this.errors.muteManager.add.userAlreadyMuted.replace('{ID}', target.id)));
                guildMutes.push(muteData);
                this.moderator.db.mutes.set(guild.id, guildMutes);
                await target.roles.add(roleID).catch((error) => { return; });
                this.moderator.emit('addMute', muteData);
                return resolve({ status: true, data: muteData });
            }
        });
    }
    /**
     * Method for getting information about the user's mute on the server
     * @param {String} targetID Discord User ID
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<{ status: Boolean, data: MuteData }>} Information about the user's mute on the server
    */
    fetch(targetID, guildID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guild = this.client.guilds.cache.get(guildID);
            if (!guild)
                return reject(new ModeratorError_1.default(this.errors.guildNotFound.replace('{gID}', guildID)));
            const guildMutes = this.moderator.db.mutes.get(guild.id);
            if (!guildMutes || guildMutes && !guildMutes.length)
                return resolve({ status: false, data: null });
            const searchTargetData = guildMutes.find(guildMute => guildMute.userID === targetID);
            if (!searchTargetData)
                return resolve({ status: true, data: null });
            return resolve({ status: true, data: searchTargetData });
        });
    }
    /**
     * Method for getting information about all user mutes on all bot servers
     * @param {String} targetID Discord User ID
     *
     * @returns {Promise<{ status: Boolean, fetchUser: Boolean, data: Array<MuteData> }>} Information about all user mutes on all bot servers
    */
    fetchAll(targetID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            let targetMutes = [];
            this.moderator.db.mutes.forEach(guildMutes => {
                const searchTargetData = guildMutes.find(guildMute => guildMute.userID === targetID);
                if (searchTargetData)
                    targetMutes.push(searchTargetData);
            });
            return resolve({ status: true, fetchUser: true, data: targetMutes.length ? targetMutes : null });
        });
    }
    /**
     * Method for unmuting a user on the server
     * @param {GuildMember} target Discord GuildMember
     *
     * @returns {Promise<Boolean>} Mute removal status for a user on the server
    */
    remove(target) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            const guild = target.guild;
            const guildMutes = this.moderator.db.mutes.get(guild.id);
            if (!this.moderator.utils.fetchClientPermissions(guild, ['ManageRoles']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (!guildMutes || guildMutes && !guildMutes.length)
                return reject(new ModeratorError_1.default(this.errors.muteManager.remove.userNotMuted.replace('{ID}', target.id)));
            const searchTargetData = guildMutes.find(guildMute => guildMute.userID === target.id);
            if (!searchTargetData)
                return reject(new ModeratorError_1.default(this.errors.muteManager.remove.userNotMuted.replace('{ID}', target.id)));
            const muteRolePosition = guild.roles.cache.get(searchTargetData.roleID)?.position || null;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            if (muteRolePosition === null)
                return resolve(false);
            if (muteRolePosition >= clientRolePosition)
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            const filteredGuildMutes = guildMutes.filter(guildMute => guildMute.userID != target.id);
            this.moderator.db.mutes.set(guild.id, filteredGuildMutes);
            await target.roles.remove(searchTargetData.roleID).catch((error) => { return; });
            this.moderator.emit('removeMute', searchTargetData);
            return resolve(true);
        });
    }
    /**
     * Method for clearing server data
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<Boolean>} Server cleanup status
    */
    clearGuild(guildID) {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guildMutes = this.moderator.db.mutes.get(guildID);
            if (!guildMutes)
                return resolve(false);
            this.moderator.db.mutes.delete(guildID);
            return resolve(true);
        });
    }
    /**
     * Method to completely clear the database
     *
     * @returns {Promise<Boolean>} Complete database cleanup status
    */
    clearAll() {
        return new Promise(async (resolve, reject) => {
            if (!this.moderator.options.muteManager)
                return reject(new ModeratorError_1.default(this.errors.muteManager.disabled));
            const dataSize = this.moderator.db.mutes.size;
            if (!dataSize)
                return resolve(false);
            this.moderator.db.mutes.deleteAll();
            return resolve(true);
        });
    }
}
module.exports = MutesManager;
