"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ModeratorError_1 = __importDefault(require("../ModeratorError"));
const ModeratorErrors_json_1 = __importDefault(require("../ModeratorErrors.json"));
/**
 * Moderator Punishments Manager
*/
class PunishmentsManager {
    client;
    moderator;
    errors;
    constructor(client, moderator) {
        /**
         * Moderator Errors JSON
         *
         * @type {Object}
         * @private
        */
        this.errors = ModeratorErrors_json_1.default;
        if (!client)
            throw new ModeratorError_1.default(this.errors.requireClient);
        if (!moderator)
            throw new ModeratorError_1.default(this.errors.requireModerator);
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
        */
        this.client = client;
        /**
         * Discord Moderator
         *
         * @type {Moderator}
         * @private
        */
        this.moderator = moderator;
    }
    /**
     * Method for kicking a user from the server
     * @param {GuildMember} target Discord GuildMember
     * @param {String} author Kick Author ID
     * @param {String} reason Kick Reason
     *
     * @returns {Promise<{ status: Boolean, data: KickData }>} Returns the punishment status and information about it
    */
    kick(target, author, reason) {
        return new Promise(async (resolve, reject) => {
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            if (!author)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'author')));
            const guild = target.guild;
            const targetRolePosition = target.roles.highest.position;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            if (!this.moderator.utils.fetchClientPermissions(guild, ['KickMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingPermissions));
            if (this.moderator.utils.fetchMemberPermissions(target, ['Administrator', 'KickMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (targetRolePosition >= clientRolePosition)
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            target.kick(reason || '').catch((error) => { return; });
            this.moderator.emit('kick', { user: target.user, guild: guild, author: author, reason: reason || null });
            return resolve({ status: true, data: { user: target.user, guild: guild, author: author, reason: reason || null } });
        });
    }
    /**
     * Method for banning a user from the server
     * @param {GuildMember} target Discord GuildMember
     * @param {String} author Ban Author ID
     * @param {String} reason Ban Reason
     *
     * @returns {Promise<{ status: Boolean, data: BanData }>} Returns the punishment status and information about it
    */
    ban(target, author, reason) {
        return new Promise(async (resolve, reject) => {
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            if (!author)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'author')));
            const guild = target.guild;
            const targetRolePosition = target.roles.highest.position;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            if (!this.moderator.utils.fetchClientPermissions(guild, ['KickMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingPermissions));
            if (this.moderator.utils.fetchMemberPermissions(target, ['Administrator', 'BanMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            if (targetRolePosition >= clientRolePosition)
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
            target.ban({ reason: reason || '' }).catch((error) => { return; });
            this.moderator.emit('ban', { user: target.user, guild: guild, author: author, reason: reason || null });
            return resolve({ status: true, data: { user: target.user, guild: guild, author: author, reason: reason || null } });
        });
    }
    /**
     * Method for unbanning a user on the server
     * @param {String} targetID Discord User ID
     * @param {String} guildID Discord Guild ID
     *
     * @returns {Promise<{ status: Boolean, data: GuildBan }>} Returns the unbanned status and information about the ban
    */
    unban(targetID, guildID) {
        return new Promise(async (resolve, reject) => {
            if (!targetID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'targetID')));
            if (!guildID)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'guildID')));
            const guild = this.client.guilds.cache.get(guildID);
            if (!this.moderator.utils.fetchClientPermissions(guild, ['BanMembers']))
                return reject(new ModeratorError_1.default(this.errors.MissingPermissions));
            const guildBans = await guild.bans.fetch();
            const searchGuildBan = guildBans.find(guildBan => guildBan.user.id === targetID);
            if (!searchGuildBan)
                return resolve({ status: false, data: null });
            guild.members.unban(searchGuildBan.user).catch((error) => { return; });
            return resolve({ status: true, data: searchGuildBan });
        });
    }
    /**
     * Method to punish the user
     * @param {GuildMember} target Discord GuildMember
     * @param {TextBasedChannel} channel Discord Text Channel
     * @param {String} author Punishment Author ID
     * @param {String} roleID Punishment Role ID
     *
     * @returns {Promise<{ status: Boolean, type: String, data: MuteData | KickData | BanData }>} Returns the punishment status and information about it
    */
    set(target, channel, author, roleID) {
        return new Promise(async (resolve, reject) => {
            if (!target)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'target')));
            if (!channel)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'channel')));
            if (!author)
                return reject(new ModeratorError_1.default(this.errors.parameterNotFound.replace('{param}', 'author')));
            const guild = target.guild;
            const reason = 'Exceeded the maximum number of warnings';
            const muteRolePosition = guild.roles.cache.get(roleID || '')?.position || null;
            const clientRolePosition = guild.members.cache.get(this.client.user.id).roles.highest.position;
            if (muteRolePosition != null) {
                if (muteRolePosition >= clientRolePosition)
                    return reject(new ModeratorError_1.default(this.errors.MissingAccess));
                switch (this.moderator.options.warnConfig.punishment) {
                    case 'mute': {
                        const muteData = { userID: target.id, guildID: guild.id, channelID: channel.id, roleID: roleID, currentTime: Date.now(), time: null, reason: reason };
                        await this.moderator.mutes.add(target, channel.id, roleID, reason);
                        return resolve({ status: true, type: this.moderator.options.warnConfig.punishment, data: muteData });
                    }
                    case 'tempmute': {
                        const muteData = { userID: target.id, guildID: guild.id, channelID: channel.id, roleID: roleID, currentTime: Date.now(), time: this.moderator.options.warnConfig.punishmentTime, reason: reason };
                        await this.moderator.mutes.temp(target, channel.id, roleID, this.moderator.options.warnConfig.punishmentTime, reason);
                        return resolve({ status: true, type: this.moderator.options.warnConfig.punishment, data: muteData });
                    }
                    case 'kick': {
                        const kickData = { user: target.user, guild: guild, author: author, reason: reason };
                        await this.kick(target, author, reason);
                        return resolve({ status: true, type: this.moderator.options.warnConfig.punishment, data: kickData });
                    }
                    case 'ban': {
                        const banData = { user: target.user, guild: guild, author: author, reason: reason };
                        await this.ban(target, author, reason);
                        return resolve({ status: true, type: this.moderator.options.warnConfig.punishment, data: banData });
                    }
                }
            }
            if (!this.moderator.utils.fetchClientPermissions(guild, ['KickMembers', 'BanMembers', 'ManageRoles']))
                return reject(new ModeratorError_1.default(this.errors.MissingAccess));
        });
    }
}
module.exports = PunishmentsManager;
