"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const ModeratorError_1 = __importDefault(require("../ModeratorError"));
const ModeratorErrors_json_1 = __importDefault(require("../ModeratorErrors.json"));
class UtilsManager {
    client;
    moderator;
    errors;
    constructor(client, moderator) {
        /**
         * Moderator Errors JSON
         *
         * @type {Object}
         * @private
        */
        this.errors = ModeratorErrors_json_1.default;
        if (!client)
            throw new ModeratorError_1.default(this.errors.requireClient);
        if (!moderator)
            throw new ModeratorError_1.default(this.errors.requireModerator);
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
        */
        this.client = client;
        /**
         * Discord Moderator
         *
         * @type {Moderator}
         * @private
        */
        this.moderator = moderator;
    }
    /**
     * Method for getting a random number
     * @param {Number} min Minimum value
     * @param {Number} max Maximum value
     *
     * @returns {Number} Random number in a given range
    */
    getRandomNumber(min, max) {
        return Math.floor(Math.random() * (max - min)) + min;
    }
    /**
     * Method for getting a random string
     * @param {Array<String>} data Array with random strings
     *
     * @returns {String} Random string
    */
    getRandomString(data) {
        return data[Math.floor(Math.random() * data.length)];
    }
    /**
     * Method for fetching Moderator options
     *
     * @returns {ModeratorOptions} Options for stable operation of the module
    */
    fetchOptions() {
        if (!this.moderator?.options || this.moderator?.options && typeof this.moderator.options != 'object') {
            this.moderator.options = {
                muteManager: true,
                warnManager: true,
                blacklistManager: true,
                dirPath: './db',
                muteConfig: {
                    name: 'mutes',
                    interval: '5s'
                },
                warnConfig: {
                    name: 'warns',
                    maxCount: 3,
                    punishment: 'ban'
                },
                blacklistConfig: {
                    name: 'blacklist',
                    punishment: 'kick'
                }
            };
        }
        else {
            if (!this.moderator.options.muteManager || typeof this.moderator.options.muteManager != 'boolean')
                this.moderator.options.muteManager = true;
            if (!this.moderator.options.warnManager || typeof this.moderator.options.warnManager != 'boolean')
                this.moderator.options.warnManager = true;
            if (!this.moderator.options.blacklistManager || typeof this.moderator.options.blacklistManager != 'boolean')
                this.moderator.options.blacklistManager = true;
            if (!this.moderator.options.dirPath || typeof this.moderator.options.dirPath != 'string')
                this.moderator.options.dirPath = './db';
            if (!this.moderator.options.muteConfig || typeof this.moderator.options.muteConfig != 'object') {
                this.moderator.options.muteConfig = {
                    name: 'mutes',
                    interval: '5s'
                };
            }
            else {
                if (!this.moderator.options.muteConfig.name || typeof this.moderator.options.muteConfig.name != 'string')
                    this.moderator.options.muteConfig.name = 'mutes';
                if (!this.moderator.options.muteConfig.name || typeof this.moderator.options.muteConfig.interval != 'string')
                    this.moderator.options.muteConfig.interval = '5s';
            }
            if (!this.moderator.options.warnConfig || typeof this.moderator.options.warnConfig != 'object') {
                this.moderator.options.warnConfig = {
                    name: 'warns',
                    maxCount: 3,
                    punishment: 'ban',
                    punishmentTime: '1d'
                };
            }
            else {
                if (!this.moderator.options.warnConfig.name || typeof this.moderator.options.warnConfig.name != 'string')
                    this.moderator.options.warnConfig.name = 'warns';
                if (!this.moderator.options.warnConfig.maxCount || typeof this.moderator.options.warnConfig.maxCount != 'number')
                    this.moderator.options.warnConfig.maxCount = 3;
                if (!this.moderator.options.warnConfig.punishment || typeof this.moderator.options.warnConfig.punishment != 'string')
                    this.moderator.options.warnConfig.punishment = 'ban';
                if (!this.moderator.options.warnConfig.punishmentTime || typeof this.moderator.options.warnConfig.punishmentTime != 'string')
                    this.moderator.options.warnConfig.punishmentTime = '1d';
            }
            if (!this.moderator.options.blacklistConfig || typeof this.moderator.options.blacklistConfig != 'object') {
                this.moderator.options.blacklistConfig = {
                    name: 'blacklist',
                    punishment: 'kick'
                };
            }
            else {
                if (!this.moderator.options.blacklistConfig.name || typeof this.moderator.options.blacklistConfig.name != 'string')
                    this.moderator.options.blacklistConfig.name = 'blacklist';
                if (!this.moderator.options.blacklistConfig.punishment || typeof this.moderator.options.blacklistConfig.punishment != 'string')
                    this.moderator.options.blacklistConfig.punishment = 'kick';
            }
        }
        return this.moderator.options;
    }
    /**
     * Method for checking member existence
     * @param {Guild} guild Discord Guild
     * @param {String | GuildMember} target Discord GuildMember or Discord User ID
     *
     * @returns {{ status: Boolean, user: User | null }} Member verification status and information about him
    */
    fetchMember(guild, target) {
        switch (typeof target) {
            case 'string': {
                const guildMember = guild.members.cache.get(target);
                if (guildMember)
                    return ({ status: true, user: guildMember.user });
                const clientMember = this.client.users.cache.get(target);
                if (clientMember)
                    return ({ status: true, user: clientMember });
                return ({ status: false, user: null });
            }
            case 'object': return ({ status: true, user: target.user });
        }
    }
    /**
     * Method for checking client permissions on the server
     * @param {Guild} guild Discord Guild
     * @param {Array<PermissionsString>} permissions Permissions Array
     *
     * @returns {Boolean} Boolean value that will indicate the presence/absence of permissions
    */
    fetchClientPermissions(guild, permissions) {
        return guild.members.cache.get(this.client.user.id).permissions.has(permissions);
    }
    /**
     * Method for checking member permissions on the server
     * @param {GuildMember} member Discord GuildMember
     * @param {Array<PermissionsString>} permissions Permissions Array
     *
     * @returns {Boolean} Boolean value that will indicate the presence/absence of permissions
    */
    fetchMemberPermissions(member, permissions) {
        return member.permissions.has(permissions);
    }
}
module.exports = UtilsManager;
