import { Client, GuildMember } from 'discord.js';

import Moderator from '../Moderator';

import { MuteData } from '../other/DatabaseData';

import ModeratorErrors from '../../src/ModeratorErrors.json';

declare class MutesManager {
    constructor(client: Client, moderator: Moderator);

    private client: Client;
    private moderator: Moderator;
    private errors: typeof ModeratorErrors;

    /**
     * Method for issuing a mute to a user
     * @param target Discord GuildMember
     * @param channelID Mute Channel ID
     * @param roleID Mute Role ID
     * @param reason Mute Reason
     * 
     * @returns Mute status and information about it
    */
    public add(target: GuildMember, channelID: string, roleID: string, reason?: string): Promise<{ status: boolean, data: MuteData }>;
    
    /**
     * Method for issuing a temporary mute to a user
     * @param target Discord GuildMember
     * @param channelID Mute Channel ID
     * @param roleID Mute Role ID
     * @param time Mute time
     * @param reason Mute Reason
     * 
     * @returns Mute status and information about it
    */
    public temp(target: GuildMember, channelID: string, roleID: string, time: string, reason?: string): Promise<{ status: boolean, data: MuteData }>;
    
    /**
     * Method for getting information about the user's mute on the server
     * @param targetID Discord User ID
     * @param guildID Discord Guild ID
     * 
     * @returns Information about the user's mute on the server
    */
    public fetch(targetID: string, guildID: string): Promise<{ status: boolean, data: MuteData }>;
    
    /**
     * Method for getting information about all user mutes on all bot servers
     * @param targetID Discord User ID
     * 
     * @returns Information about all user mutes on all bot servers
    */
    public fetchAll(targetID: string): Promise<{ status: boolean, fetchUser: boolean, data: Array<MuteData> | null }>;
    
    /**
     * Method for unmuting a user on the server
     * @param target Discord GuildMember
     * 
     * @returns Mute removal status for a user on the server
    */
    public remove(target: GuildMember): Promise<boolean>;
    
    /**
     * Method for clearing server data
     * @param guildID Discord Guild ID
     * 
     * @returns Server cleanup status
    */
    public clearGuild(guildID: string): Promise<boolean>;
    
    /**
     * Method to completely clear the database
     * 
     * @returns Complete database cleanup status
    */
    public clearAll(): Promise<boolean>;
}

export = MutesManager;